from __future__ import division
import numpy as np
from rwt import dwt, idwt, dwtaxis, idwtaxis
from rwt.wavelets import waveletCoeffs
from rwt.utilities import softThreshold, hardThreshold
import matplotlib.pyplot as plt
from scipy.misc import lena
import os

IMAGES_BASE = 'images'

def savefig(title):
    plt.savefig(os.path.join(IMAGES_BASE, title))


def tutorial1():
    img = lena()
    
    cl, ch, rl, rh = waveletCoeffs('db1')
    img_wavelet, L = dwt(img, cl, ch, L=1)
    
    plt.figure()
    plt.gray()
    plt.imshow(img_wavelet)
    savefig('lena_dwt.png')
    
    
if __name__ == '__main__':
    tutorial1()    
    plt.show()



# -*- coding: utf-8 -*-
"""
pyrwt - A cython wrapper for the Rice Wavelet Toolbox
=====================================================

The `Rice Wavelet Toolbox <http://dsp.rice.edu/software/rice-wavelet-toolbox>`_
(**RWT**) is a collection of tools for 1D and 2D wavelet and filter bank design,
analysis, and processing. RWT was developed by the DSP Group in Rice
University and is commonly used in research in the field of signal processing.

**pyrwt** is a python wrapper around RWT. It enables using RWT from the
comfort of the Python scripting language. By using pyrwt, one can compare
side by side his Python algorithm with the algorithms of researches using
RWT with other scientific ecosystems.

pyrwt is available under the 
`Simplified BSD License <http://en.wikipedia.org/wiki/BSD_license#2-clause_license_.28.22Simplified_BSD_License.22_or_.22FreeBSD_License.22.29>`_.
RWT is available under the
`BSD License <http://en.wikipedia.org/wiki/BSD_license#4-clause_license_.28original_.22BSD_License.22.29>`_.

.. codeauthor:: Amit Aides <amitibo@tx.technion.ac.il>
"""

from cyrwt import dwt, idwt, rdwt, irdwt, dwtaxis, idwtaxis



"""
rwt - Implementaion of wavelet transforms
=========================================

.. codeauthor:: Amit Aides <amitibo@tx.technion.ac.il>
"""

from __future__ import division
import numpy as np
cimport numpy as np
from rwt cimport *

DTYPEd = np.double
ctypedef np.double_t DTYPEd_t


def _prepareInputs(src_array, h, L):

    src_array = np.array(src_array, dtype=DTYPEd, copy=False, order='C')
    if src_array.ndim > 1:
        m, n = src_array.shape
    else:
        m = 1
        n = src_array.size

    h = np.array(h, dtype=DTYPEd, copy=False, order='C')
    if h.ndim > 1:
        h_row, h_col = h.shape
    else:
        h_row = 1
        h_col = h.size
    
    if h_col > h_row:
        lh = h_col
    else:    
        lh = h_row
        
    if L == None:
        #
        # Estimate L
        #
        i = n
        j = 0
        while i % 2 == 0:
            i >>= 1
            j += 1

        L = m
        i = 0
        while L % 2 == 0:
            L >>= 1
            i += 1
            
        if min(m, n) == 1:
            L = max(i, j)
        else:
            L = min(i, j)
        
    assert(L != 0, "Maximum number of levels is zero; no decomposition can be performed!")
    assert(L > 0, "The number of levels, L, must be a non-negative integer")

    #
    # Check the ROW dimension of input
    #
    if m > 1:
        mtest = m / 2.0**L
        assert(mtest == int(mtest), "The matrix row dimension must be an integer multiplication of 2**L")

    #
    # Check the COLUMN dimension of input
    #
    if n > 1:
        ntest = n / 2.0**L
        assert(ntest == int(ntest), "The matrix column dimension must be an integer multiplication of 2**L")
        
    return src_array, L, m, n, lh


def dwt(x, h0, h1, L=None):
    """        
    Computes the discrete wavelet transform y for a 1D or 2D input
    signal x using the scaling filter h0 and wavelet filter h1.

    Parameters
    ----------
    x : array-like, shape = [n] or [m, n]
        Finite length 1D or 2D signal (implicitly periodized)
    h0 : array-like, shape = [n]
        Scaling filter
    h1 : array-like, shape = [n]
        Wavelet filter
    L : integer, optional (default=None)
        Number of levels. In the case of a 1D signal, length(x) must be
        divisible by 2**L; in the case of a 2D signal, the row and the
        column dimension must be divisible by 2**L. If no argument is
        specified, a full DWT is returned for maximal possible L.

    Returns
    -------
    y : array-like, shape = [n] or [m, n]
        The wavelet transform of the signal 
        (see example to understand the coefficients)
    L : integer
	number of decomposition levels

    Examples
    --------
    2D Example:

    >>> from scipy.misc import lena
    >>> from rwt import dwt, idwt
    >>> from rwt.utilities import makeSignal
    >>> from rwt.wavelets import daubcqf
    >>> img = lena()
    >>> h0, h1 = daubcqf(4, 'min')
    >>> L = 1
    >>> y, L = dwt(img, h0, h1, L)

    See Also
    --------
    idwt, rdwt, irdwt

    """
    
    x, L, m, n, lh = _prepareInputs(x, h0, L)

    y = np.zeros_like(x, dtype=DTYPEd, order='C')
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_x = np.array(x, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_y = np.array(y, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h0 = np.array(h0, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h1 = np.array(h1, dtype=DTYPEd, copy=False, order='C').ravel()
    
    MDWT(
        <double *>np_x.data,
        m,
        n,
        <double *>np_h0.data,
        <double *>np_h1.data,
        lh,
        L,
        <double *>np_y.data
        );
        
    return y, L


def idwt(y, h0, h1, L=None):
    """
    Computes the inverse discrete wavelet transform x for a 1D or 2D
    input signal y using the scaling filter h0 and wavelet filter h1.

    Parameters
    ----------
    y : array-like, shape = [n] or [m, n]
        Finite length 1D or 2D input signal (implicitly periodized)
        (see function mdwt to find the structure of y)
    h0 : array-like, shape = [n]
        Scaling filter
    h1 : array-like, shape = [n]
        Wavelet filter
    L : integer, optional (default=None)
        Number of levels. In the case of a 1D signal, len(x) must be
        divisible by 2**L; in the case of a 2D signal, the row and the
        column dimension must be divisible by 2**L. If no argument is
        specified, a full DWT is returned for maximal possible L.

    Returns
    -------
    x : array-like, shape = [n] or [m, n]
        Periodic reconstructed signal
    L : integer
	Number of decomposition levels

    Examples
    --------
    1D Example:
    
    >>> from rwt import dwt, idwt
    >>> from rwt.utilities import makeSignal
    >>> from rwt.wavelets import daubcqf
    >>> xin = makeSignal('LinChirp', 8)
    >>> h0, h1 = daubcqf(4, 'min')
    >>> L = 1
    >>> y, L = dwt(xin, h0, h1, L)
    >>> print y
    [0.1912, 0.8821, 1.4257, 0.3101, -0.0339, 0.1001, 0.2201, 0.0000]
    >>> x, L = idwt(y, h, L)
    >>> print x
    [0.0491, 0.1951, 0.4276, 0.7071, 0.9415, 0.9808, 0.6716, 0.0000]

    See Also
    --------
    dwt, rdwt, irdwt
    """
    
    y, L, m, n, lh = _prepareInputs(y, h0, L)
    
    x = np.zeros_like(y, dtype=DTYPEd, order='C')
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_y = np.array(y, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_x = np.array(x, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h0 = np.array(h0, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h1 = np.array(h1, dtype=DTYPEd, copy=False, order='C').ravel()
    
    MIDWT(
        <double *>np_x.data,
        m,
        n,
        <double *>np_h0.data,
        <double *>np_h1.data,
        lh,
        L,
        <double *>np_y.data
        );
    
    return x, L


def rdwt(x, h0, h1, L=None):
    """
    Computes the redundant discrete wavelet transform y
    for a 1D  or 2D input signal. (Redundant means here that the
    sub-sampling after each stage is omitted.) yl contains the
    lowpass and yh the highpass components. In the case of a 2D
    signal, the ordering in yh is 
    [lh hl hh lh hl ... ] (first letter refers to row, second to
    column filtering). 

    Parameters
    ----------
    x : array-like, shape = [n] or [m, n]
        Finite length 1D or 2D signal (implicitly periodized)
    h0 : array-like, shape = [n]
        Scaling filter
    h1 : array-like, shape = [n]
        Wavelet filter
    L : integer, optional (default=None)
        Number of levels. In the case of a 1D signal, len(x) must be
        divisible by 2**L; in the case of a 2D signal, the row and the
        column dimension must be divisible by 2**L. If no argument is
        specified, a full DWT is returned for maximal possible L.

    Returns
    -------
    yl : array-like, shape = [n] or [m, n]
        Lowpass component
    yh : array-like, shape = [n] or [m, n]
        Highpass component
    L : integer
	number of decomposition levels
        
    See Also
    --------
    dwt, idwt, irdwt

    Warnings
    --------
    min(x.shape)/2**L should be greater than len(h)

    """

    x, L, m, n, lh = _prepareInputs(x, h0, L)
    
    yl = np.zeros_like(x, dtype=DTYPEd, order='C')
    if min(m, n) == 1:
        yh = np.zeros((m, L*n), dtype=DTYPEd, order='C')
    else:
        yh = np.zeros((m, 3*L*n), dtype=DTYPEd, order='C')
	
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_x = np.array(x, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h0 = np.array(h0, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h1 = np.array(h1, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_yl = np.array(yl, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_yh = np.array(yh, dtype=DTYPEd, copy=False, order='C').ravel()
    
    MRDWT(
        <double *>np_x.data,
        m,
        n,
        <double *>np_h0.data,
        <double *>np_h1.data,
        lh,
        L,
        <double *>np_yl.data,
        <double *>np_yh.data
        );
    
    return yl, yh, L
    

def irdwt(yl, yh, h0, h1, L=None):
    """
    Computes the inverse redundant discrete wavelet
    transform x  for a 1D or 2D input signal. (Redundant means here
    that the sub-sampling after each stage of the forward transform
    has been omitted.) yl contains the lowpass and yl the highpass
    components as computed, e.g., by mrdwt. In the case of a 2D
    signal, the ordering in
    yh is [lh hl hh lh hl ... ] (first letter refers to row, second
    to column filtering).  

    Parameters
    ----------
    yl : array-like, shape = [n] or [m, n]
        Lowpass component
    yh : array-like, shape = [n] or [m, n]
        Highpass component
    h0 : array-like, shape = [n]
        Scaling filter
    h1 : array-like, shape = [n]
        Wavelet filter
    L : integer
	number of levels. In the case of a 1D signal, 
	len(yl) must  be divisible by 2**L;
	in the case of a 2D signal, the row and
	the column dimension must be divisible by 2**L.        
   
    Returns
    -------
    x : array-like, shape = [n] or [m, n]
        Finite length 1D or 2D signal
    L : integer
	number of decomposition levels

    See Also
    --------
    dwt, idwt, rdwt

    Warnings
    --------
    min(yl.shape)/2**L should be greater than len(h)

    """

    h0 = h0 / 2
    h1 = h1 / 2
    
    yl, L, m, n, lh = _prepareInputs(yl, h0, L)
    
    yh = np.array(yh, copy=False, order='C')
    mh, nh = yh.shape
    
    #
    # check for consistency of rows and columns of yl, yh
    #
    if min(m, n) > 1:
        assert(m == mh and nh == 3*n*L, "Dimensions of first two input matrices not consistent!")
    else:
        assert(m == mh and nh == n*L, "Dimensions of first two input vectors not consistent!")

    x = np.zeros_like(yl, dtype=DTYPEd, order='C')
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_x = np.array(x, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h0 = np.array(h0, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h1 = np.array(h1, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_yl = np.array(yl, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_yh = np.array(yh, dtype=DTYPEd, copy=False, order='C').ravel()
    
    MIRDWT(
        <double *>np_x.data,
        m,
        n,
        <double *>np_h0.data,
        <double *>np_h1.data,
        lh,
        L,
        <double *>np_yl.data,
        <double *>np_yh.data
        );
    
    return x, L
    
    
def _prepareInputs2(src_array, h, L, axis):

    src_array = np.array(src_array, dtype=DTYPEd, copy=False, order='C')
    
    assert(axis >= 0 and axis <= src_array.ndim, "Wrong axis value")
    
    shape = src_array.shape
    prod_h = int(np.prod(shape[:axis]))
    stride = int(np.prod(shape[axis+1:]))
    n = shape[axis]
    
    h = np.array(h, dtype=DTYPEd, copy=False, order='C').ravel()
    lh = h.size
        
    if L == None:
        #
        # Estimate L
        #
        i = n
        L = 0
        while i % 2 == 0:
            i >>= 1
            L += 1

    assert(L != 0, "Maximum number of levels is zero; no decomposition can be performed!")
    assert(L > 0, "The number of levels, L, must be a non-negative integer")

    return src_array, L, n, stride, prod_h, lh


def dwtaxis(x, h0, h1, axis=0, L=None):
    """        
    Computes the discrete wavelet transform over signal x along a specified
    axis using the scaling filter h0, and wavelet filter h1.

    Parameters
    ----------
    x : array-like, shape = Arbitrary dimension
        Input signal
    h0 : array-like, shape = [n]
        Scaling filter
    h1 : array-like, shape = [n]
        Wavelet filter
    axis : integer, optional (default=0)
        The axis of x for which to perform the transform.
    L : integer, optional (default=None)
        Number of levels. In the case of a 1D signal, length(x) must be
        divisible by 2**L; in the case of a 2D signal, the row and the
        column dimension must be divisible by 2**L. If no argument is
        specified, a full DWT is returned for maximal possible L.

    Returns
    -------
    y : array-like, shape = Same dimension of x.
        The wavelet transform of the input signal 
    L : integer
	number of decomposition levels

    See Also
    --------
    idwt, rdwt, irdwt, idwtaxis

    """
    
    
    x, L, n, stride, prod_h, lh = _prepareInputs2(x, h0, L, axis)
    
    y = np.empty(x.shape, dtype=DTYPEd, order='C')
    
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_x = np.array(x, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h0 = np.array(h0, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h1 = np.array(h1, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_y = y.ravel()
    
    DWTAXIS(
        <double *>np_x.data,
        n,
	prod_h,
	stride,
        <double *>np_h0.data,
        <double *>np_h1.data,
        lh,
        L,
        <double *>np_y.data
        );
    
    return y, L


def idwtaxis(y, h0, h1, axis=0, L=None):
    """        
    Computes the inverse discrete wavelet transform over signal x along a
    specified axis using the scaling filter h0, and wavelet filter h1.

    Parameters
    ----------
    y : array-like, shape = Arbitrary dimension
        Input signal
    h0 : array-like, shape = [n]
        Scaling filter
    h1 : array-like, shape = [n]
        Wavelet filter
    axis : integer, optional (default=0)
        The axis of x for which to perform the transform.
    L : integer, optional (default=None)
        Number of levels. In the case of a 1D signal, length(x) must be
        divisible by 2**L; in the case of a 2D signal, the row and the
        column dimension must be divisible by 2**L. If no argument is
        specified, a full DWT is returned for maximal possible L.

    Returns
    -------
    x : array-like, shape = Same dimension of x.
        The inverse wavelet transform of the input signal 
    L : integer
	number of decomposition levels

    See Also
    --------
    idwt, rdwt, irdwt, dwtaxis

    """
    
    
    y, L, n, stride, prod_h, lh = _prepareInputs2(y, h0, L, axis)
    
    x = np.empty(y.shape, dtype=DTYPEd, order='C')
    
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_y = np.array(y, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h0 = np.array(h0, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_h1 = np.array(h1, dtype=DTYPEd, copy=False, order='C').ravel()
    cdef np.ndarray[DTYPEd_t, ndim=1]  np_x = x.ravel()
    
    IDWTAXIS(
        <double *>np_x.data,
        n,
	prod_h,
	stride,
        <double *>np_h0.data,
        <double *>np_h1.data,
        lh,
        L,
        <double *>np_y.data
        );
    
    return x, L





cdef extern from "rwt.h":

    void MDWT(
        double *x,
        int m,
        int n,
        double *h0,
        double *h1,
        int lh,
        int L,
        double *y
        )
        
    void MIDWT(
        double *x,
        int m,
        int n,
        double *h0,
        double *h1,
        int lh,
        int L,
        double *y
        )
        
    void MRDWT(
        double *x,
        int m,
        int n,
        double *h0,
        double *h1,
        int lh,
        int L,
        double *yl,
        double *yh
        )

    void MIRDWT(
        double *x,
        int m,
        int n,
        double *h0,
        double *h1,
        int lh,
        int L,
        double *yl,
        double *yh
        )

    void DWTAXIS(
        double *x,
        int n,
        int prod_h,
        int stride,
        double *h0,
        double *h1,
        int lh,
        int L,
        double *y
        )

    void IDWTAXIS(
        double *x,
        int n,
        int prod_h,
        int stride,
        double *h0,
        double *h1,
        int lh,
        int L,
        double *y
        )



"""
utilities - Several utilities useful when using pyrwt
=====================================================

.. codeauthor:: Amit Aides <amitibo@tx.technion.ac.il>
"""

from __future__ import division
import numpy as np

def hardThreshold(y, thld):
    """
    Hard thresholds the input signal y with the threshold value
    thld.

    Parameters
    ----------
    y : array-like, shape =  Arbitrary dimension
        Finite length signal (implicitly periodized)
    thld : float
        Value by which to threshold the input signal
        
    Returns
    -------
    x : array-like, shape = Same dimension of y
        Hard thresholded output ``x = (abs(y)>thld)*y``

    Examples
    --------
    
    >>> from rwt.utilities import makeSignal, hardThreshold
    >>> y = makeSignal('WernerSorrows', 8)
    >>> print hardThreshold(y, thld=1)
    [1.5545, 5.3175, 0, 1.6956, -1.2678, 0, 1.7332, 0]

    See Also
    --------
    softThreshold
    """
    
    x = np.zeros_like(y)
    ind = np.abs(y) > thld
    x[ind] = y[ind]
    
    return x


def softThreshold(y, thld):
    """
    Soft thresholds the input signal y with the threshold value
    thld.

    Parameters
    ----------
    y : array-like, shape = Arbitrary dimension
        Finite length signal (implicitly periodized)
    thld : float
        Value by which to threshold the input signal
        
    Returns
    -------
    x : array-like, shape = Same dimension as y
        Soft thresholded output x = ``sign(y)(abs(y)-thld)_+``

    Examples
    --------
    >>> from rwt.utilities import makeSignal, hardThreshold
    >>> y = makeSignal('Doppler', 8)
    >>> print softThreshold(y, thld=0.2)
    [0, 0, 0, -0.0703, 0, 0.2001, 0.0483, 0]

    See Also
    --------
    hardThreshold
    """
    
    x = np.abs(y) - thld
    x[x<0] = 0
    x[y<0] = -x[y<0]

    return x


def makeSignal(signal_name='AllSig', N=512):
    """
    Creates artificial test signal identical to the
    standard test signals proposed and used by D. Donoho and I. Johnstone
    in WaveLab (- a matlab toolbox developed by Donoho et al. the statistics
    department at Stanford University).

    Parameters
    ----------
    signal_name : string, optional (default='AllSig')
        Name of the desired signal. Supported values:
            * 'AllSig' (Returns a list with all the signals)
            * 'HeaviSine'
            * 'Bumps'
            * 'Blocks'
            * 'Doppler'
            * 'Ramp'
            * 'Cusp'
            * 'Sing'
            * 'HiSine'
            * 'LoSine'
            * 'LinChirp'
            * 'TwoChirp'
            * 'QuadChirp'
            * 'MishMash'
            * 'Werner Sorrows' (Heisenberg)
            * 'Leopold' (Kronecker)
            
    N : integer, optional (default=512)
        Length in samples of the desired signal

    Returns
    -------
    x : array/list of arrays, shape = [N]

    References
    ----------
    WaveLab can be accessed at
    www_url: http://playfair.stanford.edu/~wavelab/
    Also see various articles by D.L. Donoho et al. at
    web_url: http://playfair.stanford.edu/
    """

    t = np.linspace(1, N, N)/N
    signals = []
    
    if signal_name in ('HeaviSine', 'AllSig'):
        y = 4 * np.sin(4*np.pi*t) - np.sign(t - 0.3) - sign(0.72 - t)
        
        signals.append(y)
    
    if signal_name in ('Bumps', 'AllSig'):
        pos = np.array([ .1, .13, .15, .23, .25, .40, .44, .65, .76, .78, .81])
        hgt = np.array([ 4,  5,   3,   4,  5,  4.2, 2.1, 4.3,  3.1, 5.1, 4.2])
        wth = np.array([.005, .005, .006, .01, .01, .03, .01, .01,  .005, .008, .005])
        y = np.zeros_like(t)
        for p, h, w in zip(pos, hgt, wth):
            y += h/(1 + np.abs((t - p)/w))**4

        signals.append(y)
    
    if signal_name in ('Blocks', 'AllSig'):
        pos = np.array([ .1, .13, .15, .23, .25, .40, .44, .65, .76, .78, .81])
        hgt = np.array([ 4,  -5,   3,   -4,  5,  -4.2, 2.1, 4.3,  -3.1, 2.1, -4.2])
        y = np.zeros_like(t)
        for p, h in zip(pos, hgt):
            y += (1 + np.abs(t - p))*h/2
            
        signals.append(y)

    if signal_name in ('Doppler', 'AllSig'):
        y = np.sqrt(t * (1-t)) * np.sin((2*np.pi*1.05) / (t+.05))
                
        signals.append(y)

    if signal_name in ('Ramp', 'AllSig'):
        y = t.copy()
        y[t >= .37] -= 1
                
        signals.append(y)
    
    if signal_name in ('Cusp', 'AllSig'):
        y = np.sqrt(np.abs(t - 0.37))
                
        signals.append(y)

    if signal_name in ('Sing', 'AllSig'):
        k = np.floor(N * .37)
        y = 1 / np.abs(t - (k+.5)/N)
                
        signals.append(y)

    if signal_name in ('HiSine', 'AllSig'):
        y = np.sin(N*0.6902*np.pi*t)
                
        signals.append(y)

    if signal_name in ('LoSine', 'AllSig'):
        y = np.sin(N*0.3333*np.pi*t)
                
        signals.append(y)

    if signal_name in ('LinChirp', 'AllSig'):
        y = np.sin(N*0.125*np.pi*t*t)
                
        signals.append(y)

    if signal_name in ('TwoChirp', 'AllSig'):
        y = np.sin(N*np.pi*t*t) + np.sin(N*np.pi/3*t*t)
                
        signals.append(y)

    if signal_name in ('QuadChirp', 'AllSig'):
        y = np.sin(N*np.pi/3*t*t*t)
                
        signals.append(y)

    if signal_name in ('MishMash', 'AllSig'):
        #
        # QuadChirp + LinChirp + HiSine
        #
        y = np.sin(N*np.pi/3*t*t*t) + np.sin(N*0.125*np.pi*t*t) + np.sin(N*0.6902*np.pi*t)
                
        signals.append(y)

    if signal_name in ('WernerSorrows', 'AllSig'):
        y = np.sin(N/2*np.pi*t*t*t)
        y += np.sin(N*0.6902*np.pi*t)
        y += np.sin(N*np.pi*t*t)
        
        pos = np.array([.1, .13, .15, .23, .25, .40, .44, .65, .76, .78, .81])
        hgt = np.array([4, 5, 3, 4, 5, 4.2, 2.1, 4.3, 3.1, 5.1, 4.2])
        wth = np.array([.005, .005, .006, .01, .01, .03, .01, .01, .005, .008, .005])
        
        for p, h, w in zip(pos, hgt, wth):
            y += h/(1 + np.abs((t - p)/w))**4

        signals.append(y)

    if signal_name in ('Leopold', 'AllSig'):
        y = (t == np.floor(.37 * N)/N).astype(np.float)
                
        signals.append(y)

    if len(signals) == 1:
        return signals[0]

    return signals


# -*- coding: utf-8 -*-
"""
pyrwt: A cython wrapper for the Rice Wavelet Toolbox, written in Cython.

Copyright (C) 2012 Amit Aides
Author: Amit Aides <amitibo@tx.technion.ac.il>
URL: <http://bitbucket.org/amitibo/pyrwt>
License: See attached license file
"""
from setuptools import setup
from setuptools.extension import Extension
from distutils.sysconfig import get_python_lib
from Cython.Distutils import build_ext
import numpy as np
import os.path
import sys


NAME = 'pyrwt'
PACKAGE_NAME = 'rwt'
VERSION = '0.2.1'
DESCRIPTION = 'A cython wrapper for the Rice Wavelet Toolbox'
LONG_DESCRIPTION = """
pyrwt is a python package that uses cython to wrap the
RICE Wavelet Toolbox. It is fast and enables comparing
the results of your python algorithm with the results
of algorithms that use the Matalb toolbox.
"""
AUTHOR = 'Amit Aides'
EMAIL = 'amitibo@tx.technion.ac.il'
URL = "http://bitbucket.org/amitibo/pyrwt"
KEYWORDS = ["wavelets", "wavelet transform", "DWT"]
LICENSE = 'BSD'
CLASSIFIERS = [
    'License :: OSI Approved :: BSD License',
    'Development Status :: 3 - Alpha',
    'Topic :: Scientific/Engineering'
]
IPOPT_ICLUDE_DIRS=[]
IPOPT_ICLUDE_DIRS += [np.get_include()]

def main():
    setup(
        name=NAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        author=AUTHOR,
        author_email=EMAIL,
        url=URL,
        keywords=KEYWORDS,
        classifiers=CLASSIFIERS,
        license=LICENSE,
        packages=[PACKAGE_NAME],
        cmdclass = {'build_ext': build_ext},
        ext_modules = [
            Extension(
                PACKAGE_NAME + '.' + 'cyrwt',
                [
                    'src/cyrwt.pyx',
                    'src/dwtaxis.c',
                    'src/mdwt_r.c',
                    'src/midwt_r.c',
                    'src/mrdwt_r.c',
                    'src/mirdwt_r.c',
                ],
                include_dirs=IPOPT_ICLUDE_DIRS
            )
        ],
    )


if __name__ == '__main__':
    main()
